"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.scrapeBetaChangesets = exports.scrapeArchivedChangesets = exports.getBetaChangeset = exports.getArchivedChangeset = exports.getUnityChangeset = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const jsdom_1 = require("jsdom");
const unityChangeset_1 = require("./unityChangeset");
const UNITY_ARCHIVE_URL = 'https://unity3d.com/get-unity/download/archive';
const UNITY_BETA_URL = 'https://unity3d.com/beta';
const getDocumentFromUrl = async (archiveUrl) => {
    const response = await node_fetch_1.default(archiveUrl);
    const html = await response.text();
    return new jsdom_1.JSDOM(html).window.document;
};
const getUnityChangeset = async (version) => {
    const match = version.match(/^\d{4}\.\d+\.\d+(a|b|f)\d+$/);
    switch (match === null || match === void 0 ? void 0 : match[1]) {
        case 'f':
            return await exports.getArchivedChangeset(version);
        case 'a':
            return await exports.getBetaChangeset(version, 'alpha');
        case 'b':
            return await exports.getBetaChangeset(version, 'beta');
        default:
            throw Error('The given version was not supported');
    }
};
exports.getUnityChangeset = getUnityChangeset;
const getArchivedChangeset = async (version) => {
    const versions = await exports.scrapeArchivedChangesets();
    return versions.filter(c => c.version === version)[0];
};
exports.getArchivedChangeset = getArchivedChangeset;
const getBetaChangeset = async (version, channel) => {
    const document = await getDocumentFromUrl(`https://unity3d.com/unity/${channel}/${version}`);
    return Array.from(document.querySelectorAll('a[href]'))
        .map(a => a.getAttribute('href'))
        .filter(href => unityChangeset_1.UnityChangeset.isValid(href))
        .map(href => unityChangeset_1.UnityChangeset.createFromHref(href))[0];
};
exports.getBetaChangeset = getBetaChangeset;
const scrapeArchivedChangesets = async () => {
    const document = await getDocumentFromUrl(UNITY_ARCHIVE_URL);
    return Array.from(document.querySelectorAll('a[href]'))
        .map(a => a.getAttribute('href'))
        .filter(href => unityChangeset_1.UnityChangeset.isValid(href))
        .map(href => unityChangeset_1.UnityChangeset.createFromHref(href));
};
exports.scrapeArchivedChangesets = scrapeArchivedChangesets;
const scrapeBetaChangesets = async () => {
    const document = await getDocumentFromUrl(UNITY_BETA_URL);
    const betas = new Set();
    Array.from(document.querySelectorAll('a[href]'))
        .map((a) => a.getAttribute('href'))
        .filter((href) => /^\/(alpha|beta)\/\d{4}\.\d(a|b)$/.test(href))
        .forEach((href) => betas.add(href));
    const downloads = new Set();
    for (const beta of betas) {
        // [beta page] e.g. 'https://unity3d.com/beta/2020.2b'
        const betaPage = await getDocumentFromUrl(`https://unity3d.com${beta}`);
        Array.from(betaPage.querySelectorAll('a[href]'))
            .map((a) => a.getAttribute('href'))
            // [filter] e.g. '/unity/beta/2020.2.0b13'
            .filter((href) => /^\/unity\/(alpha|beta)\/\d{4}\.\d+\.\d+(a|b)\d+$/.test(href))
            .forEach((href) => downloads.add(href));
    }
    const hrefs = new Set();
    for (const download of downloads) {
        // [download page] e.g. https://unity3d.com/unity/beta/2020.2.0b13
        const downloadPage = await getDocumentFromUrl(`https://unity3d.com${download}`);
        Array.from(downloadPage.querySelectorAll('a[href]'))
            .map((a) => a.getAttribute('href'))
            // [filter] e.g. 'unityhub://2020.2.0b13/655e1a328b90'
            .filter((href) => unityChangeset_1.UnityChangeset.isValid(href))
            .forEach((href) => hrefs.add(href));
    }
    return Array.from(hrefs)
        .map(href => unityChangeset_1.UnityChangeset.createFromHref(href));
};
exports.scrapeBetaChangesets = scrapeBetaChangesets;
//# sourceMappingURL=index.js.map